﻿using ProjectAirBomber.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace ProjectAirBomber.CollectionGenericObjects;

/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class ListGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Список объектов, которые храним
    /// </summary>
    private readonly List<T?> _collection;
    /// <summary>
    /// Максимально допустимое число объектов в списке
    /// </summary>
    private int _maxCount;
    public int Count => _collection.Count;

    public CollectionType GetCollectionType => CollectionType.List;

    public int MaxCount
    {
        get
        {
            return _collection.Count;
        }
        set
        {
            if (value > 0)
            {
                _maxCount = value;
            }
        }
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    public ListGenericObjects()
    {
        _collection = new();
    }
    public T? Get(int position)
    {
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null) throw new ObjectNotFoundException();
        return _collection[position];
    }
    public int Insert(T obj)
    {
        if (Count + 1 > _maxCount) throw new CollectionOverflowException(Count);
        _collection.Add(obj);
        return Count;
    }
    public int Insert(T obj, int position)
    {
        if (Count + 1 > _maxCount) throw new CollectionOverflowException(Count);
        if (position < 0 || position > Count) throw new PositionOutOfCollectionException(position);
        _collection.Insert(position, obj);
        return position;
    }
    public T? Remove(int position)
    {
        if (position < 0 || position > Count) throw new PositionOutOfCollectionException(position);
        T? temp = _collection[position];
        _collection.RemoveAt(position);
        return temp;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Count; ++i)
        {
            yield return _collection[i];
        }
    }
}